@@ -25,12 +25,14 @@
 #include "app-autostart.h"
 #include "app-resumed.h"
 #include "logout-dialog.h"
+#include "consolekit.h"
 #include "power-manager.h"
 #include "gdm.h"
 #include "gconf.h"
 #include "gsm.h"
 #include "session.h"
 #include "xsmp.h"
+#include "dbus.h"
 #include "util.h"
 
 static void append_default_apps       (GsmSession *session,
@@ -66,6 +68,8 @@ static void client_disconnected        (GsmClient *client,
 					gpointer   data);
 
 struct _GsmSession {
+  char *name;
+
   /* Startup/resumed apps */
   GSList *apps;
   GHashTable *apps_by_name;
@@ -118,6 +122,8 @@ gsm_session_new (gboolean failsafe)
   char **autostart_dirs;
   int i;
 
+  session->name = NULL;
+
   session->clients = NULL;
   session->condition_clients = NULL;
 
@@ -149,12 +155,36 @@ out:
   return session;
 }
 
+/**
+ * gsm_session_set_name:
+ * @session: session instance 
+ * @name: name of the session 
+ *
+ * Sets the name of a running session.
+ **/
+void
+gsm_session_set_name (GsmSession *session, const char *name)
+{
+  if (session->name)
+    g_free (session->name);
+
+  session->name = g_strdup (name);
+}
+
 static void
 append_app (GsmSession *session, GsmApp *app)
 {
-  const char *basename = gsm_app_get_basename (app);
-  GsmApp *dup = g_hash_table_lookup (session->apps_by_name, basename);
+  const char *basename;
+  GsmApp *dup;
 
+  basename = gsm_app_get_basename (app);
+  if (basename == NULL)
+    {
+      g_object_unref (app);
+      return;
+    }
+
+  dup = g_hash_table_lookup (session->apps_by_name, basename);
   if (dup)
     {
       /* FIXME */
@@ -234,6 +264,7 @@ append_default_apps (GsmSession *session, char **autostart_dirs)
         }
 
       g_free (desktop_file);
+      g_key_file_free (key_file);
     }
 
   g_slist_foreach (default_apps, (GFunc) g_free, NULL);
@@ -444,6 +475,9 @@ end_phase (GsmSession *session)
 
   if (session->phase < GSM_SESSION_PHASE_RUNNING)
     start_phase (session);
+
+  if (session->phase == GSM_SESSION_PHASE_RUNNING)
+    gsm_dbus_emit_session_running ();
 }
 
 static void
@@ -846,11 +880,43 @@ initiate_shutdown (GsmSession *session)
     }
 }
 
+static void
+do_request_reboot (GsmConsolekit *consolekit)
+{
+  if (gsm_consolekit_can_restart (consolekit)) 
+    {
+      gdm_set_logout_action (GDM_LOGOUT_ACTION_NONE);
+      gsm_consolekit_attempt_restart (consolekit);
+    } 
+  else 
+    {
+      gdm_set_logout_action (GDM_LOGOUT_ACTION_REBOOT);
+    }
+}
+
+static void
+do_request_shutdown (GsmConsolekit *consolekit)
+{
+  if (gsm_consolekit_can_stop (consolekit)) 
+    {
+      gdm_set_logout_action (GDM_LOGOUT_ACTION_NONE);
+      gsm_consolekit_attempt_stop (consolekit);
+    } 
+  else 
+    {
+      gdm_set_logout_action (GDM_LOGOUT_ACTION_SHUTDOWN);
+    }
+}
+
 static void
 session_shutdown (GsmSession *session)
 {
+  GsmConsolekit *consolekit;
   GSList *cl;
 
+  /* Emit session over signal */
+  gsm_dbus_emit_session_over ();
+
   /* FIXME: do this in reverse phase order */
   for (cl = session->clients; cl; cl = cl->next)
     gsm_client_die (cl->data);
@@ -858,11 +924,19 @@ session_shutdown (GsmSession *session)
   switch (session->logout_response_id) 
     {
     case GSM_LOGOUT_RESPONSE_SHUTDOWN:
-      gdm_set_logout_action (GDM_LOGOUT_ACTION_SHUTDOWN);
+      consolekit = gsm_get_consolekit ();
+
+      do_request_shutdown (consolekit);
+
+      g_object_unref (consolekit);
       break;
 
     case GSM_LOGOUT_RESPONSE_REBOOT:
-      gdm_set_logout_action (GDM_LOGOUT_ACTION_REBOOT);
+      consolekit = gsm_get_consolekit ();
+
+      do_request_reboot (consolekit);
+
+      g_object_unref (consolekit);
       break;
 
     default:
